package com.app.markeet.adapter;

import android.content.Context;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.markeet.AppConfig;
import com.app.markeet.data.Constant;
import com.app.markeet.data.SharedPref;
import com.app.markeet.databinding.ItemLoadingBinding;
import com.app.markeet.databinding.ItemProductBinding;
import com.app.markeet.model.Product;
import com.app.markeet.utils.Tools;

import java.util.ArrayList;
import java.util.List;

public class AdapterProduct extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private final int VIEW_ITEM = 1;
    private final int VIEW_PROG = 0;

    private List<Product> items = new ArrayList<>();
    private boolean loading = true;
    private String status;
    private int page = 0;
    private AdapterListener<Product> listener;
    public void setListener(AdapterListener<Product> listener) {
        this.listener = listener;
    }

    private Context ctx;

    // Provide a suitable constructor (depends on the kind of dataset)
    public AdapterProduct(Context context, RecyclerView view, int page) {
        this.page = page;
        ctx = context;
        lastItemViewDetector(view);
    }

    public class OriginalViewHolder extends RecyclerView.ViewHolder {
        ItemProductBinding binding;

        public OriginalViewHolder(@NonNull ItemProductBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }


    public static class ProgressViewHolder extends RecyclerView.ViewHolder {
        ItemLoadingBinding binding;

        public ProgressViewHolder(@NonNull ItemLoadingBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;
        if (viewType == VIEW_ITEM) {
            ItemProductBinding v = ItemProductBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new OriginalViewHolder(v);
        } else {
            ItemLoadingBinding v = ItemLoadingBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new ProgressViewHolder(v);
        }
        return vh;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder holder, final int pos) {
        int position = holder.getBindingAdapterPosition();
        Object obj = items.get(position);
        if (holder instanceof OriginalViewHolder) {
            ItemProductBinding v = ((OriginalViewHolder) holder).binding;
            final Product p = (Product) obj;
            v.name.setText(p.name);

            // handle discount view
            if (p.price_discount > 0) {
                v.price.setText(Tools.getFormattedPrice(p.price_discount, ctx));
                v.priceStrike.setText(Tools.getFormattedPrice(p.price, ctx));
                v.priceStrike.setPaintFlags(v.priceStrike.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
                v.priceStrike.setVisibility(View.VISIBLE);
                v.saleBadge.setVisibility(View.VISIBLE);
            } else {
                v.price.setText(Tools.getFormattedPrice(p.price, ctx));
                v.priceStrike.setVisibility(View.GONE);
                v.saleBadge.setVisibility(View.GONE);
            }
            Tools.displayImageOriginal(ctx, v.image, Constant.getURLimgProduct(p.image));
            v.lytParent.setOnClickListener(view -> {
                if (listener == null) return;
                listener.onClick(view, null, p, position);
            });
        } else {
            final ItemLoadingBinding v = ((ProgressViewHolder) holder).binding;
            v.progressLoading.setVisibility(status == null ? View.VISIBLE : View.GONE);
            v.statusLoading.setVisibility(status == null ? View.GONE : View.VISIBLE);

            if (status == null) return;
            v.statusLoading.setText(status);
            v.statusLoading.setOnClickListener(view -> {
                setLoaded();
                onLoadMore();
            });
        }
    }

    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return items.size();
    }

    @Override
    public int getItemViewType(int position) {
        return this.items.get(position) != null ? VIEW_ITEM : VIEW_PROG;
    }

    public void insertData(List<Product> items) {
        setLoaded();
        int positionStart = getItemCount();
        int itemCount = items.size();
        this.items.addAll(items);
        notifyItemRangeInserted(positionStart, itemCount);
    }

    public void setLoaded() {
        loading = false;
        for (int i = 0; i < getItemCount(); i++) {
            if (items.get(i) == null) {
                items.remove(i);
                notifyItemRemoved(i);
            }
        }
    }

    public void setLoading() {
        if (getItemCount() != 0) {
            this.items.add(null);
            notifyItemInserted(getItemCount() - 1);
            loading = true;
        }
    }

    public void setLoadingOrFailed(String status) {
        setLoaded();
        this.status = status;
        this.items.add(null);
        notifyItemInserted(getItemCount() - 1);
        loading = true;
    }

    public void resetListData() {
        this.items = new ArrayList<>();
        notifyDataSetChanged();
    }

    boolean scrollDown = false;

    private void lastItemViewDetector(RecyclerView recyclerView) {
        if (recyclerView.getLayoutManager() instanceof GridLayoutManager) {
            final GridLayoutManager layoutManager = (GridLayoutManager) recyclerView.getLayoutManager();
            recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                    super.onScrollStateChanged(recyclerView, newState);
                    if (newState == RecyclerView.SCROLL_STATE_IDLE || !scrollDown) return;
                    int lastPos = layoutManager.findLastVisibleItemPosition();
                    boolean bottom = lastPos >= getItemCount() - page;
                    if (!loading && bottom && listener != null) {
                        onLoadMore();
                    }
                }

                @Override
                public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                    super.onScrolled(recyclerView, dx, dy);
                    scrollDown = dy > 0;
                }
            });

            layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
                @Override
                public int getSpanSize(int position) {
                    int type = getItemViewType(position);
                    int spanCount = layoutManager.getSpanCount();
                    return type == VIEW_ITEM ? 1 : spanCount;
                }
            });
        }
    }

    private void onLoadMore() {
        int current_page = getItemCount() / page;
        listener.onLoadMore(current_page);
        loading = true;
        status = null;
    }

}